
// 2006-xx by Stefan Clara
// Slave Controller

////////////////////////////////////////////////////////////////////////////////

#include <avr/interrupt.h>
#include <stdlib.h>
#include <Init.h>
#include <math.h>

////////////////////////////////////////////////////////////////////////////////

// ################
// ## Konstanten ##
// ################

#define anzahlmessungen 3															//2^anzahlmessungen Messungen
#define fprescaler 30//25

#define ReglerFrequenzfaktor 6

#define PosShift 0																// brauchbare Daten ADC

#define Stepperschritte 200														//Halbschritt
#define Getriebezaehne 120

#define RadProSchritt ((2* M_PI)/ (Getriebezaehne* Stepperschritte))
#define RadProADCBit (M_PI/ (0x9252>> PosShift))
#define Steppergrundfrequenz 59252

#define StartADCshort 0x80
#define CalibrateADCexternal 0x48

#define TWIMaxData 11
#define MaxMoves 40

#define USART_Buffer_maxdata 2

// #####################
// ## TWI Statusbytes ##
// #####################
#define START 0x08																//Start
#define RSTART 0x10																//repeated Start
#define DATA_LOST 0x38															//Data lost
#define BUS_ERROR 0x00															//Bus error
#define NO_RELEVANT_INFORMATION 0xF8											//no relevant information

//MT
#define MT_SLA_ACK 0x18															//Slave W AK
#define MT_SLA_NACK 0x20														//Slave W NAK
#define MT_DATA_ACK 0x28														//Data W AK
#define MT_DATA_NACK 0x30														//Data W NAK

//MR
#define MR_SLA_ACK 0x40															//Slave R AK
#define MR_SLA_NACK 0x48														//Slave R NAK
#define MR_DATA_ACK 0x50														//Data R AK
#define MR_DATA_NACK 0x58														//Data R NAK

//SR
#define SR_SLA_ACK 0x60															//Slave R AK
#define SR_SLA_NACK 0x68														//Slave R AK
#define SR_GCALL_ACK 0x70														//General call AK
#define SR_DATA_NACK 0x78														//Data R NAK
#define SR_PREVADR_ACK 0x80														//Previously addressed R AK
//#define SR_SLA_NACK 0x88														//Slave R NAK
#define SR_DATA_ACK 0x90														//Data R AK
//#define SR_DATA_NACK 0x98														//Data R NAK
#define SR_STOP 0xA0

//ST
#define ST_GCALL_ACK 0xA8														//Slave_R R ACK
#define ST_DATA_NACK 0xB0														//Slave_R R ACK
#define ST_DATA_ACK 0xB8														//Data R ACK
#define ST_DARA_NACK 0xC0														//DATA R NAK
#define ST_LDATA_ACK 0xC8														//Last Data transmittet


// #################
// ## TWI Befehle ##
// #################
#define DataOK 0xAA																// -> Master sends 1Byte Accept Code
#define DataERROR 0xFF															// -> Master sends 1Byte Error Code
#define DataSend 0xA1

//Data Accept Codes
#define AdressAcceptet 0x10
#define CommandAcceptet 0x20
#define Parameterset 0x30
#define MovesCleared 0x40
#define MoveAdded 0x41
#define SlaveCommandSend 0x50
#define MovesStarted 0x60
#define NextMoveStartet 0x61
#define MovesStoped 0x62

//Error Codes
#define AdressUnknown 0x10
#define CommandUnknown 0x20
#define USARTBufferFull 0x30
#define SlaveNotOnline 0x40
#define MoveBufferFull 0x50
#define TWIError 0x60

//Adresses
#define MasterAdress 0x01
#define firstSlaveAdress 0x80													// -> 0. Slave -> General Call Bit 0
#define lastSlaveAdress 0x8E													// -> 7. Slave

// Protokoll
#define Online 0x10																// -> Master sends 3Bytes (0xFF 0x4D 0xFF); Slave sends 3Bytes (0xFF 0x53 0xFF)
#define setParameter 0x20														// -> not implemented
#define getParameter 0x21														// -> Slave sends 2Bytes Current Max, Min
#define ClearMoves 0x60															// -> Clears Buffer
#define getMoveNumber 0x61														// -> sends Number of Moves in Buffer
#define getActivMove 0x62														// -> sends Number of Moves active Move
#define AddMove 0x63															// -> Master: +4Bytes: Speed L, H; Steps L, H; Slave: +5Bytes: Speed L, H; Steps L, H; Parameter
#define getPosition 0x70														// -> Slave sends 2Bytes position L, H
#define StartMoves 0xE0
#define StopMoves 0xE1

// #######################
// ## Stepper Parameter ##
// #######################
#define EN 7
#define HALF 6
#define CW 5
#define TON 4

// ##########################
// ## ADC Status Parameter ##
// ##########################
#define cal 0
#define convert 1
#define get 2
#define ndata 3

///////////////////////////////////////////////////////////////////////////////

// #####################
// ## Datenstrukturen ##
// #####################
struct s_SPI {
	int8_t data[2];
	uint8_t DataCount;
} SPIBus;

struct s_ADC {
	volatile uint8_t Status;															//0 Kalibrierung, 1 Konvertierung Gestartet, 2 Daten Holen, 3 neue Daten vorhanden
	int32_t data;
	int16_t ADCValue;
	uint16_t DataCount;
	uint16_t maxdata;
	uint8_t InterruptStatus;
} ADC1;

struct s_USART_Buffer {
	uint8_t data[USART_Buffer_maxdata];
	uint8_t index;
	uint8_t datacount;
	uint8_t NewData;
};

struct s_usart {
	struct s_USART_Buffer USART_receive_Buffer;
	struct s_USART_Buffer USART_transmit_Buffer;
} usart;

struct s_TWI_Data {
	uint8_t data[TWIMaxData];
	uint8_t datacount;
	uint8_t numberOfData;
};

struct s_TWI {
	struct s_TWI_Data TWI_transmit_Data;
	struct s_TWI_Data TWI_receive_Data;

	uint8_t Adress;
	uint8_t mode;
} TWIBus;


struct s_Move {
	float start;
	float end;
	uint16_t sollspeed;
} Moves[MaxMoves];

struct s_stepper {
	float ist;																// ADCVal+ offsetmessung
	float soll;
	uint16_t speed;

	float rpsspeed;

	float PosMax;
	float PosMin;

	float offsetmessung;

	uint8_t parameter;															//7 Enable(0), 6 Half(1), 5 Richtung(0), 4 Bewegen(0)

	uint8_t StromMax;
	uint8_t StromMin;

	uint16_t sollspeedcounter;
	uint16_t istspeedcounter;

	uint16_t StepperSpeedMax;
	uint16_t StepperSpeedMin;

	struct s_Move *Move;

	float K_b;	
	float R_r;																	//I Anteil des Reglers
	float K_i;																	//Aufintegrierte Abweichung
	
} stepper;

struct s_BeRe {
	float zi;	

	float k_b;
	float k_r;
	float k_i;

	float Ta;

	float v;

} BeRe;

///////////////////////////////////////////////////////////////////////////////

// #######################
// ## Globale Variablen ##
// #######################

uint8_t MoveCounter= 0x00;
uint8_t NumberOfMoves= 0x00;

uint8_t RunPinStatus= 0x00;

uint8_t controllerstatus= 1;

uint8_t ADCIntCount= 0;

uint8_t Regelungscount= 0;

uint8_t PCPortStatus= 0;

volatile uint8_t step;

uint8_t LEDCounter= 0x00;
uint8_t LedCounterMax= 0x00;
///////////////////////////////////////////////////////////////////////////////

// ###########
// ## USART ##
// ###########
void USART_transmit(uint8_t c) {												//Zeichen vom Controller an den PC senden
	while (!(UCSR0A & (1<< UDRE0)));
	UDR0 = c;
} //USART_transmit

ISR(SIG_USART_RECV) {															//Funktion zum Empfangen eines Zeichens von PC an Controller

} //SIGNAL(SIG_UART_RECV)

// ################
// ## Status LED ##
// ################
void changeStatus(uint8_t status){
	switch (status) {
	case 1:
		LedCounterMax= 0x00;
		break;
	case 2:
		LedCounterMax= 0xFF;
		break;
	case 3:
		LedCounterMax= 0x01;
		break;
	default:
		break;
	}
} //changeStatus


ISR(SIG_OVERFLOW0) {											//Counter fr Status LED
	
	ADC1.InterruptStatus= EIMSK& (1<< INT0);					//ADC InterruptStatus speichern
	EIMSK&= ~(1<< INT0);
															
	switch (LedCounterMax) {
		case 0x00:
			PORTD|= (1<< PD6);
			break;
		case 0xFF:
			PORTD&= ~(1<< PD6);
			break;
		default:
			LEDCounter++;
			if (LEDCounter>= LedCounterMax) {
				PORTD= (PIND^ (1<< PD6));
				LEDCounter= 0x00;
			}
			break;
	}
	EIMSK= ADC1.InterruptStatus;								//ADC InterruptStatus zurckschreiben
}

// #########################
// ## Einheitenumrechnung ##
// #########################
float RadProSecToSpeed(float *speed, uint8_t *parameter) {
	uint16_t res;
	
	if (*speed > 0) {
		*parameter= (1<< EN)| (1<< HALF)| (1<< CW)| (1<< TON);
		res= 0xFFFF- Steppergrundfrequenz* RadProSchritt/ *speed;
	} else {
		*parameter= (1<< EN)| (1<< HALF)| (1<< TON);
		res= 0xFFFF- Steppergrundfrequenz* RadProSchritt/ (-*speed);		
	}
	
	return res;	
}

// ##############
// ## Regelung ##
// ##############
void getSettings(float *sollPos, uint16_t *speed, float *istPos) 
{	

	float u[2]= {*istPos, *sollPos};
	float zin= BeRe.zi+ (u[0]- u[1])* BeRe.Ta* ReglerFrequenzfaktor;

	stepper.rpsspeed= (u[1]* BeRe.v)- (-BeRe.k_r* u[0]- BeRe.k_i* zin);
	
	float rspeed;

	rspeed= RadProSecToSpeed(&stepper.rpsspeed, &stepper.parameter);
	
	if (rspeed< stepper.StepperSpeedMax) {											//Anti winde up
		BeRe.zi= zin;											
		stepper.speed= trunc(rspeed);
	} else {
		stepper.speed= stepper.StepperSpeedMax;
	}		

//	float b= *sollPos;	
	

/*	uint8_t *a;

	a= istPos;

	//USART_transmit('B');	
	USART_transmit(a[3]);
	USART_transmit(a[2]);
	USART_transmit(a[1]);
	USART_transmit(a[0]);*/
	
}

// ########################
// ## Stepper Funktionen ##
// ########################
void setCurrent(uint8_t Current) {												// 0x00 -> 0.0110V -> 0.02A
	if (Current<= 0x54)	{														// 0x14 -> 0.2448V -> 0.48A
		OCR2B= Current;															// 0x29 -> 0.4896V -> 0.96A
	} else {																	// 0x4F -> 0,9180V -> 1.80A
		OCR2B= 0x54;															// 0x54 -> 0,9792V -> 1.92A
	}																			// 0xFF -> 2,9690V -> 5.82A
}																				// MaximalStrom abgesichert

int addMove(struct s_Move Move) {
	if (NumberOfMoves< MaxMoves) {
		Moves[NumberOfMoves]= Move;
		NumberOfMoves++;
	} else {
		return 1;
	}
	return 0;
}

void setParameters(uint8_t parameter) {
	if (parameter& (1<< EN))													//Stepper Einschalten
		PORTD|= (1<< PD7);
	else
		PORTD&= ~(1<< PD7);

	if (parameter& (1<< HALF))													//Halbschritt
		PORTC|= (1<< PC0);
	else
		PORTC&= ~(1<< PC0);

	if (parameter& (1<< CW))													//Drehrichtung
		PORTB|= (1<< PB0);
	else
		PORTB&= ~(1<< PB0);

	if (parameter& (1<< TON)) {													//Stepper starten
		setCurrent(stepper.StromMax);											//Strom auf Betriebsstrom
		TCNT1= 0x0000;
	} else {
		setCurrent(stepper.StromMin);												//Strom auf Ruhestrom
	}
}

void setNextMove() {
	if (MoveCounter< NumberOfMoves) {
		stepper.Move= &Moves[MoveCounter];

		if (stepper.Move->start< stepper.PosMin) {
			stepper.Move->start= stepper.PosMin;
		}
		if (stepper.Move->start> stepper.PosMax) {
			stepper.Move->start= stepper.PosMax;
		}

		if (stepper.Move->end< stepper.PosMin) {
			stepper.Move->end= stepper.PosMin;
		}
		if (stepper.Move->end> stepper.PosMax) {
			stepper.Move->end= stepper.PosMax;
		}

		stepper.soll= stepper.Move->start;
		MoveCounter++;
	} else {
		stepper.Move= NULL;
		NumberOfMoves= 0;
		MoveCounter= 0;
	}
}

ISR(SIG_OUTPUT_COMPARE1A) {														//Clock generator fr ist und soll
	step= 1;
	TCCR1B= 0x00;
}

// #########
// ## TWI ##
// #########
ISR(SIG_2WIRE_SERIAL) {															//TWI Interrupt	
	ADC1.InterruptStatus= EIMSK& (1<< INT0);					//ADC InterruptStatus speichern
	EIMSK&= ~(1<< INT0);														//ADC Interrupt ausschalten
	switch (TWSR & 0xF8) {
		//Slave Reciever
		case SR_SLA_ACK:
			TWIBus.TWI_receive_Data.datacount= 0;
			TWCR= (1<< TWINT)| (1<< TWEA)| (1<< TWEN)| (1<< TWIE);
			break;
		case SR_PREVADR_ACK:
			TWIBus.TWI_receive_Data.data[TWIBus.TWI_receive_Data.datacount]= TWDR;			
			TWIBus.TWI_receive_Data.datacount++;
			switch (TWIBus.TWI_receive_Data.data[0]) {
				case Online:													//TO DO Daten Senden
					TWIBus.TWI_receive_Data.datacount= 0;

					TWIBus.TWI_transmit_Data.data[0]= 0xFF;
					TWIBus.TWI_transmit_Data.data[1]= 0x53;
					TWIBus.TWI_transmit_Data.data[2]= 0xFF;
					TWIBus.TWI_transmit_Data.datacount= 0;
					TWIBus.TWI_transmit_Data.numberOfData= 3;

					break;
				case setParameter:
					if (TWIBus.TWI_receive_Data.datacount== 3) {
						stepper.StromMax= TWIBus.TWI_receive_Data.data[1];
						stepper.StromMin= TWIBus.TWI_receive_Data.data[2];
						TWIBus.TWI_receive_Data.datacount= 0;

						TWIBus.TWI_transmit_Data.data[0]= DataOK;
						TWIBus.TWI_transmit_Data.data[1]= Parameterset;
						TWIBus.TWI_transmit_Data.datacount= 0;
						TWIBus.TWI_transmit_Data.numberOfData= 2;
					}
					break;
				case getParameter:											//TO DO Daten Senden
					TWIBus.TWI_receive_Data.datacount= 0;

					TWIBus.TWI_transmit_Data.data[0]= stepper.StromMax;
					TWIBus.TWI_transmit_Data.data[1]= stepper.StromMin;
					TWIBus.TWI_transmit_Data.datacount= 0;
					TWIBus.TWI_transmit_Data.numberOfData= 2;
					break;
				case ClearMoves:
					MoveCounter= 0x00;
					NumberOfMoves= 0x00;
					TWIBus.TWI_receive_Data.datacount= 0;

					TWIBus.TWI_transmit_Data.data[0]= DataOK;
					TWIBus.TWI_transmit_Data.data[1]= MovesCleared;
					TWIBus.TWI_transmit_Data.datacount= 0;
					TWIBus.TWI_transmit_Data.numberOfData= 2;
					break;
				case getMoveNumber:										//TO DO Daten Senden
					TWIBus.TWI_receive_Data.datacount= 0;

					TWIBus.TWI_transmit_Data.data[0]= NumberOfMoves;
					TWIBus.TWI_transmit_Data.datacount= 0;
					TWIBus.TWI_transmit_Data.numberOfData= 1;
					break;
				case getActivMove:											//TO DO Daten Senden
					TWIBus.TWI_receive_Data.datacount= 0;

					TWIBus.TWI_transmit_Data.data[0]= MoveCounter;
					TWIBus.TWI_transmit_Data.datacount= 0;
					TWIBus.TWI_transmit_Data.numberOfData= 1;
					break;
				case AddMove:
					if (TWIBus.TWI_receive_Data.datacount== 11) {			// >= 6 war vorher
						uint8_t *data;										// Zeiger wird verwendet um gesamte Struct zu empfangen
						struct s_Move Move;									// kein Fehler !
						data= &Move;
						uint8_t i;

						for (i= 0; i< 10; i++)
							data[i]= TWIBus.TWI_receive_Data.data[i+ 1];
						
						
						if (addMove(Move)) {
							TWIBus.TWI_receive_Data.datacount= 0;

							TWIBus.TWI_transmit_Data.data[0]= DataERROR;
							TWIBus.TWI_transmit_Data.data[1]= MoveBufferFull;
							TWIBus.TWI_transmit_Data.datacount= 0;
							TWIBus.TWI_transmit_Data.numberOfData= 2;
						} else {						
							TWIBus.TWI_receive_Data.datacount= 0;

							TWIBus.TWI_transmit_Data.data[0]= DataOK;
							TWIBus.TWI_transmit_Data.data[1]= MoveAdded;
							TWIBus.TWI_transmit_Data.datacount= 0;
							TWIBus.TWI_transmit_Data.numberOfData= 2;
						}						
					}
					break;
				case getPosition:											//TO DO Daten Senden
					TWIBus.TWI_receive_Data.datacount= 0;
					
					uint8_t *data;
					data= &stepper.ist;

					TWIBus.TWI_transmit_Data.data[0]= (data[0]);
					TWIBus.TWI_transmit_Data.data[1]= (data[1]);
					TWIBus.TWI_transmit_Data.data[1]= (data[2]);
					TWIBus.TWI_transmit_Data.data[1]= (data[3]);
					TWIBus.TWI_transmit_Data.datacount= 0;
					TWIBus.TWI_transmit_Data.numberOfData= 4;
					break;
				default:
					TWIBus.TWI_receive_Data.datacount= 0;

					TWIBus.TWI_transmit_Data.data[0]= DataERROR;
					TWIBus.TWI_transmit_Data.data[1]= CommandUnknown;
					TWIBus.TWI_transmit_Data.datacount= 0;
					TWIBus.TWI_transmit_Data.numberOfData= 2;
					break;
			}


			TWCR= (1<< TWINT)| (1<< TWEA)| (1<< TWEN)| (1<< TWIE);
			break;

		//Slave Transmitter
		case ST_GCALL_ACK:

			TWIBus.TWI_transmit_Data.datacount= 0;
			TWDR= TWIBus.TWI_transmit_Data.data[TWIBus.TWI_transmit_Data.datacount];
			TWIBus.TWI_transmit_Data.datacount++;


			if (TWIBus.TWI_transmit_Data.datacount>= TWIBus.TWI_transmit_Data.numberOfData)
				TWCR= (1<< TWINT)| (1<< TWEN)| (1<< TWIE);						//Bus abgeben
			else
				TWCR= (1<< TWINT)| (1<< TWEA)| (1<< TWEN)| (1<< TWIE);
			break;
		case ST_DATA_ACK:

			TWDR= TWIBus.TWI_transmit_Data.data[TWIBus.TWI_transmit_Data.datacount];
			TWIBus.TWI_transmit_Data.datacount++;

			if (TWIBus.TWI_transmit_Data.datacount>= TWIBus.TWI_transmit_Data.numberOfData)
				TWCR= (1<< TWINT)| (1<< TWEN)| (1<< TWIE);						//Bus abgeben
			else
			  TWCR= (1<< TWINT)| (1<< TWEA)| (1<< TWEN)| (1<< TWIE);
			break;

		//Fehler oder unbehandelter Fall
		default:
			//USART_transmit(0xAF);
			//USART_transmit(TWSR & 0xF8);

			//if ((TWSR & 0xF8) == 0xC0) 
			//PORTD|= (1<< PD7);


			TWCR= (1<< TWINT)| (1<< TWEA)| (1<< TWEN)| (1<< TWIE);
		break;
	}
	EIMSK= ADC1.InterruptStatus;								//ADC InterruptStatus zurckschreiben
} //SIGNAL(SIG_TWI_RECV)

// #########
// ## SPI ##
// #########
void SPI_transmit(uint8_t data) {
	SPDR= data;
}

ISR(SIG_SPI) {																	//SPI Transfer Complete	
	EIMSK&= ~(1<< INT0);														//ADC Interrupt ausschalten
		
	switch (ADC1.Status& ((1<< ndata)| (1<< get)| (1<< convert)| (1<< cal))) {
		case 0x01:			
			SPI_transmit(0x00);						
			break; 
		case 0x02:
			SPI_transmit(0x00);			
			ADC1.Status= (1<< get);
			EIMSK|= (1<< INT0);															//ADC Interrupt einschalten
			break;		
		case 0x04:			
			switch (SPIBus.DataCount) {
				case 0:
					SPIBus.data[1]= SPDR;											//Array umgekehrt einlesen
					SPIBus.DataCount= 1;					
					SPI_transmit(0x00);
					break;
				case 1:		
					SPIBus.data[0]= SPDR;
					SPIBus.DataCount= 0;
					PORTB|= (1<< PB2);												//CS fr ADC lschen

					int16_t *temp;													//Zeiger wird verwendet um aus 2 8 bit Variablen direkt
					temp= SPIBus.data;												//eine 16 bit Variable zu lesen

					if (EICRA & (1<< ISC00)) {
						ADC1.data+= *temp;
					} else {
						ADC1.data-= *temp;
					}

					ADC1.DataCount++;				

					//USART_transmit('A');
					//USART_transmit(HIGH(*temp));
					//USART_transmit(LOW(*temp));
					

					if (ADC1.DataCount>= ADC1.maxdata) {
						ADC1.ADCValue= (((int16_t)(ADC1.data >> (anzahlmessungen)))>> PosShift);						
						
						ADC1.Status= (1<< ndata);															
						
						ADC1.DataCount= 0;
						ADC1.data= 0;				
					} else {
						EIMSK|= (1<< INT0);	
						ADC1.Status= 0x00;					
					}						
					break;
				default:					
					break;
			}
			break;
		default:			
			break;
	}
}


// #########
// ## ADC ##
// #########
void ADC_StartCalibration() {	
	EIMSK&= ~(1<< INT0);											//Interrupt auschalten und Messung Starten

	PORTB&= ~(1<< PB2);
	ADC1.Status= (1<< cal);	
	SPI_transmit(CalibrateADCexternal);
}

void Start_ADC() {
	SPIBus.DataCount= 0;
	PORTB&= ~(1<< PB2);		
	SPI_transmit(StartADCshort);
	ADC1.Status= (1<< convert);
}

ISR(SIG_INTERRUPT0) {														//ADC Messung starten

	if (ADC1.Status== 0x00) {		

		ADCIntCount++;

		if (ADCIntCount>= fprescaler) {
			EIMSK&= ~(1<< INT0);											//Interrupt auschalten und Messung Starten		
			EICRA^= (1<< ISC00);											//Umschalten zwischen steigendere und fallender Flanke
	

			ADCIntCount= 0x00;
			
			ADC1.Status= (1<< convert);					

			Start_ADC();
		}
	}
}


ISR(SIG_PIN_CHANGE2) {	
	ADC1.InterruptStatus= EIMSK& (1<< INT0);					//ADC InterruptStatus speichern
	EIMSK&= ~(1<< INT0);															//ADC Interrupt ausschalten
	
	uint8_t ToggledPins= (PCPortStatus^ PIND);
 
 	if (ToggledPins& (1<< PD4)) {
		if (RunPinStatus!= (PIND& (1<< PD4))) {
			RunPinStatus^= (1<< PD4);
			setNextMove();
		}
	}
	
	if (ToggledPins& (1<< PD5)) {	
		switch (ADC1.Status& ((1<< ndata)| (1<< get)| (1<< convert)| (1<< cal))) {
			case 0x01:
				if (!(PIND & (1<< PD5))) {	
					PORTB|= (1<< PB2);			
					ADC1.Status= 0x00;								//Status bereit setzen
					ADCIntCount= 0x00;								//Interruptcounter auf 0 setzen
					SPIBus.DataCount= 0;
				
					Start_ADC();

					ADC1.InterruptStatus|= (1<< INT0);								//Ext ADC Interrupt einschalten	
									
				}
				break;
			default:
				break;
		}
	} 
	
	PCPortStatus= PIND;

	EIMSK= ADC1.InterruptStatus;								//ADC InterruptStatus zurckschreiben
}


// ##########
// ## INIT ##
// ##########
void BeRe_init() {
	BeRe.zi= 0;

BeRe.k_b= -0.010000;
BeRe.k_r= -1.386667;
BeRe.k_i= -0.010000;
BeRe.Ta= 0.004808;
BeRe.v= 1.386667;

}

void ADC_init() {
	ADC1.maxdata= (1<< anzahlmessungen);

	ADC1.Status= 0x00;

	ADC1.data= 0x00000000;
	ADC1.ADCValue= 0x0000;
	ADC1.DataCount= 0x0000;

	ADC1.InterruptStatus= 0;

	ADC_StartCalibration();	
}

void Stepper_init() {
	stepper.ist= 0;

	switch (TWIBus.Adress) {
		case 0x80:
			stepper.soll= 0;
			break;
		case 0x82:
			stepper.soll= -M_PI/ 2;
			break;
		case 0x84:
			stepper.soll= M_PI/ 2;
			break;
		case 0x86:
			stepper.soll= 0;
			break;
		case 0x88:
			stepper.soll= M_PI/ 2*0;
			break;
		default:
			stepper.soll= 0;
			break;
	}

	stepper.rpsspeed= 0;

	stepper.offsetmessung= 0;
	stepper.speed= 0;

	stepper.PosMax= 2;
	stepper.PosMin= -1;

	stepper.parameter= (1<< HALF);													//7Enable(0), 6Half(1), 5Richtung(0), 4 Bewegen(0)
	
	stepper.sollspeedcounter= 0xFFFF;
	stepper.istspeedcounter= 0xFFFF;

	stepper.StepperSpeedMax= 0xFFB0;
	stepper.StepperSpeedMin= 0x0000;

	stepper.Move= NULL;

	stepper.StromMax= 0x20;
	stepper.StromMin= 0x08;
}

void Vars_init() {
	SPIBus.DataCount= 0;
	RunPinStatus= (PIND& (1<< PD4));

	TWIBus.TWI_transmit_Data.datacount= 0x00;
	TWIBus.TWI_transmit_Data.numberOfData= 0x00;

	TWIBus.TWI_receive_Data.datacount= 0x00;
	TWIBus.TWI_receive_Data.numberOfData= 0x00;

	PCPortStatus= PIND;
}

void Init() {
	//Ausgnge festlegen
	DDRB= (1<< DDB0)| (1<< DDB1)| (1<< DDB2)| (1<< DDB3)| (1<< DDB5);
	DDRD= (1<< DDD1)| (1<< DDD3)| (1<< DDD6)| (1<< DDD7);
	DDRC= (1<< DDC0);

	//Pullups Einschalten
	PORTB|= (1<< PB2);
	PORTC|= (1<< PC1)| (1<< PC2)| (1<< PC3);
	
	SPI_init();

	USART_init();
	USART_transmit('S');
	
	TWIBus.Adress= ((~PINC)& ((1<< PC1)| (1<< PC2)| (1<< PC3)))| 0x80;
	TWI_init(TWIBus.Adress);
	USART_transmit(TWIBus.Adress);
	USART_transmit('S');

	PCINT_init();

	EXTINT_init();	

	ADC_init();

	PWM_init();

	Stepper_init();

	Vars_init();

	BeRe_init();

	sei();
} //Init()


// ##########
// ## main ##
// ##########
int main(void) {
	Init();
	
	//PORTD|= (1<< PD6);

	changeStatus(controllerstatus);

	// Testmove

/*	struct s_Move Move;

	Move.start= 0;
	Move.end= 1;
	Move.sollspeed= 0xFFA0;

	addMove(Move);

	Move.start= 1;
	Move.end= -1;
	Move.sollspeed= 0xFFA0;

	addMove(Move);

	Move.start= -1;
	Move.end= 0;
	Move.sollspeed= 0xFFA0;

	addMove(Move);

	setNextMove();*/

	// Testmove

	//BeRe.z[0]	

	for (;;) {																//Endlosschleife	

		if (step) {			
			stepper.sollspeedcounter--;
			stepper.istspeedcounter--;

			if (stepper.Move != NULL) {			
				if (stepper.sollspeedcounter< stepper.Move->sollspeed) {
					if (stepper.Move->start< stepper.Move->end) {							//Bewegungsrichtugn
						if (stepper.soll<= stepper.Move->end) {
							stepper.soll+= RadProSchritt;
						}// else setNextMove();		
					} else {
						if (stepper.soll>= stepper.Move->end) {					
							stepper.soll-= RadProSchritt;									
						}// else setNextMove();
					}
					stepper.sollspeedcounter= 0xFFFF;
				}
			}

			if (stepper.istspeedcounter>= stepper.speed) {								//Schrittmotorenansteuersignal erzeugen
				PORTB|= (1<< PB1);
			} else {
				PORTB&= ~(1<< PB1);		
		
				stepper.istspeedcounter= 0xFFFF;		
			}

			step= 0;			
			TCNT1= 0x0000;
			TCCR1B= (1<< CS10);	
		}

		if (ADC1.Status== (1<< ndata)) {


			stepper.ist*= (1+ BeRe.k_b);
			
			stepper.ist+= -BeRe.k_b* (ADC1.ADCValue* RadProADCBit+ stepper.offsetmessung);						
						
			//USART_transmit('A');
			//USART_transmit(HIGH(ADC1.ADCValue>> PosShift));
			//USART_transmit(LOW(ADC1.ADCValue>> PosShift));
														
							
			Regelungscount++;
						
			if (Regelungscount>= ReglerFrequenzfaktor) {

				getSettings(&stepper.soll, &stepper.speed, &stepper.ist);		

				setParameters(stepper.parameter);							
						
				Regelungscount= 0;		

			//	uint8_t *a;

			//	a= &stepper.ist;
		
				//USART_transmit('B');	
			//	USART_transmit(a[3]);
			//	USART_transmit(a[2]);
			//	USART_transmit(a[1]);
			//	USART_transmit(a[0]);	
			}

			EIMSK|= (1<< INT0);	
			ADC1.Status= 0x00;	
		}	
			
	} 

} //main
